/*
 * Copyright 1995 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <VpfUnidraw/vpfclasses.h>
#include <VpfUnidraw/vpfcomps.h>
#include <VpfUnidraw/vpfeditor.h>
#include <VpfUnidraw/vpftilesel.h>

#include <Vpf/vpfdb.h>
#include <Vpf/vpflib.h>
#include <Vpf/vpfcov.h>
#include <Vpf/vpffeature.h>
#include <Vpf/vpffclass.h>

#include <MapUnidraw/mapcmds.h>
#include <MapUnidraw/mapcomps.h>
#include <MapUnidraw/mapviews.h>
#include <MapUnidraw/mapstates.h>

#include <OverlayUnidraw/ovviewer.h>
#include <Unidraw/iterator.h>
#include <Unidraw/manips.h>
#include <Unidraw/selection.h>

#include <InterViews/cursor.h>
#include <InterViews/event.h>
#include <InterViews/window.h>
#include <IV-2_6/InterViews/rubrect.h>

#include <IV-2_6/_enter.h>

#include <string.h>

/*****************************************************************************/

ClassId TileSelectTool::GetClassId () { return TILESELECT_TOOL; }

boolean TileSelectTool::IsA (ClassId id) {
    return TILESELECT_TOOL == id || Tool::IsA(id);
}

TileSelectTool::TileSelectTool (ControlInfo* m) : Tool(m) { }
Tool* TileSelectTool::Copy () { return new TileSelectTool(CopyControlInfo()); }

Manipulator* TileSelectTool::CreateManipulator (
    Viewer* v, Event& e, Transformer*
) {
    VpfEditor* ed = (VpfEditor*)v->GetEditor();
    VpfDatabase* db = ed->GetMapDatabase();
    MapLibState* libstate = ed->libstate();
    VpfLibrary* lib = db->library(libstate->maplib());
    if (lib && lib->tiled()) {
        Selection* ts = ed->tiles();
        if (!e.shift) {
	  if (ts)
	    ts->Clear();
	}
	RubberRect* rr = new RubberRect(nil, nil, e.x, e.y, e.x, e.y);
	return new DragManip(v, rr);
    }
    else
	return nil;
}

inline void fixpath(char* path) {
    int i = 0;
    while (path[i]) {
	if (path[i]=='\\')
	    path[i]='/';
	else {
	    int c = path[i];
	    c = tolower(path[i]);
	    path[i] = c;
	}
	i++;
    }
}

Command* TileSelectTool::InterpretManipulator (Manipulator* m) {
    if (m) {
	Viewer* v = m->GetViewer();
	DragManip* dm = (DragManip*) m;
	RubberRect* rr = (RubberRect*) dm->GetRubberband();
	Coord l, r, b, t;

	rr->GetCurrent(l, b, r, t);

	// intersect rubber rect with current library\'s tileref
	// and show intersecting tiles
	v->GetEditor()->GetWindow()->cursor(hourglass);
	VpfEditor* ed = (VpfEditor*)v->GetEditor();
	VpfDatabase* db = ed->GetMapDatabase();
	MapLibState* libstate = ed->libstate();
	VpfLibrary* lib = db->library(libstate->maplib());
	if (lib && lib->tiled()) {
	    VpfCoverage* cov = lib->coverage("tileref");
	    if (cov) {
		VpfFeatureClass* fc = cov->feature_class("tileref");
		if (fc) {
		    MapDbView* dbview = ed->GetMapDbView();
		    MapLibView* libview = dbview->FindLibView(lib);
		    if (!libview) {  // show tileref
			ShowTiles(ed, lib);
			libview = dbview->FindLibView(lib);
		    }
		    if (libview) {
			MapCovView* covview = libview->FindCovView(cov);
			if (!covview) {  // show tileref
			    ShowTiles(ed, lib);
			    covview = libview->FindCovView(cov);
			}
			if (covview) {
			    MapFClassView* fcview = covview->FindFClassView(fc);
			    if (!fcview) {  // show tileref
				ShowTiles(ed, lib);
				fcview = covview->FindFClassView(fc);
			    }
			    if (fcview) {
				((VpfFClassView*)fcview)->IgnoreNone(true);
				Selection* tileviews =
				    fcview->ViewsIntersecting(l, b, r, t);
				((VpfFClassView*)fcview)->IgnoreNone(false);
#if 1
				Iterator it;
				for (tileviews->First(it);
				     !tileviews->Done(it);
				     tileviews->Next(it))
				  {
				    long count;
				    MapFeatureView* tv = (MapFeatureView*)tileviews->GetView(it);
				    VpfFeature* tile = (VpfFeature*)
				      ((MapFeatureComp*)tv->GetOverlayComp())->feature();
				    char* tname = (char*)tile->named_row_element("TILE_NAME", NULL, &count);
				    fixpath(tname);
				    VpfExtent* ext = tile->extent();
				    printf("%s %f %f %f %f\n", tname, ext->x1, ext->y1, ext->x2, ext->y2);
				    free(tname);
				  }
				printf("\n");
#endif
				Selection* ts = ed->tiles();
				if (ts)
				  tileviews->Merge(ts);
				ed->tiles(tileviews);
			    }
			}
		    }
		}
	    }
	}
	v->GetEditor()->GetWindow()->cursor(arrow);
    }
    return nil;
}

void TileSelectTool::ShowTiles(Editor* ed, MapLibrary* lib) {
    MapShowCmd* showcmd = new MapShowCmd(ed, false);
    showcmd->SetLibName(lib->name());
    showcmd->SetCovName("tileref");
    showcmd->SetFClassName("tileref");
    showcmd->Execute();
}
